/* $Id: Design.java,v 1.75 2016/10/07 00:01:27 mit3 Exp $ */
/* Design.java v0.6 */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.text.DecimalFormat;
import java.util.*;

public class Design extends JPanel
    implements ActionListener, AdjustmentListener, ChangeListener,
	       MouseListener, MouseMotionListener, Runnable
{
    private JButton jbPm;   // >
    private JButton jbMm;   // <
    private JButton jbGrap;
    private JButton jbWire;
    private JButton jbCent;
    private JButton jbTune;
    private JButton jbType;
    private JButton jbSet;
    private JButton jbSave;
    private JButton jbOnoff; // D.on/off
    private JButton jbSof;   // W.on/off
    private JButton jbDebug;
    private JButton jbItype;
    private JLabel jlUnit;
    private JLabel jlBeat;
    private JTextField jtLeng;   // length
    private JTextField jtBant;   // gauge
    private JTextField jtCopp;   // copper
    private JTextField jtData;
    private JTextField jtFile;
    private JComboBox jcValue;   // wire_data
    private JScrollBar jsbWid;
    private JSlider jslBeat;
    private JSpinner jspBass;
    private JSpinner jspWire;

    final static int KB = 88;
    final static int Oct = 12;
    final static int Oct2 = Oct*2;
    final static int A49 = Oct*4;  // 49-1;
    final static int C88 = KB-1;
    final static int Ocu = Oct+1;
    final static int BHW = 28;     // ϥޡ bass
    final static int MHW = 30;     // " middle
    final static int HHW = 30;     // " high
    final static int BEND = BHW-1; // 
    final static int BSTT = 0;     // 
    final static int AA = 440;
    final static int WSTT = 1400;  // ̥μˡ
    final static int SBS = 800;    // 
    final static int SMD = 110;    // 
    final static int OFF = 0;
    final static int ON = 1;
    final static int GRAP = 0;
    final static int WIRE = 1;
    final static int CENT = 2;
    final static double MLOG10 = Math.log(10);
    final static double E = 2.15*100000; //  (쥹)[N/mm^2]
    final static String[] Grwi = {"Graph","Wire"};
    final static String[] Offon = {"OFF","ON"};

    private ScaleCalculation SC;
    private TuningKeyboard TK;
    private TuningFormula2 TF2;
    private PaintGraph PG;
    private Interval I;
    private Bante B;
    private Thread thr;
    private DecimalFormat df1 = new DecimalFormat("#0.0###");
    private DecimalFormat df2 = new DecimalFormat("#0.0#");
    private Wire[] TG;
    private minMax[] MX;

    boolean CALCsw;   // calc.sw.
    boolean UPGPsw;   // up/gp
    boolean MOUSWsw;  // mouse sw.
    boolean VALUEsw;  // value sw.
    boolean STRINGsw; // string sw.
    boolean CURVEsw;  // curve/straight
    boolean TUNEsw;   // tune/clear
    boolean DEBUG = false; // for DEBUG
    int DISP;
    int STT;
    int END;
    int Vlist;
    int WIDE;    // ̥ѹ
    int XBS;     // 
    int XMD;     // 
    String Error;

    public Design() {
	I = new Interval();
	B = new Bante();
        PG = new PaintGraph();
	TF2 = new TuningFormula2();

	init_minmax();
	buildLayout();
	setenabled();

        Border border = new BevelBorder(BevelBorder.RAISED);
        border = new TitledBorder(border, "Design");
        setBorder(border);

	init_set();
        init_wire();
    }
    public static void main(String argv[]) {
	Design D2 = new Design();

        JFrame f = new JFrame();
        f.getContentPane().add(D2);
        f.pack();
        f.setSize(600, 570);
        f.setVisible(true);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
    public void buildLayout() {
	setLayout(new BorderLayout());
        Border bo = new BevelBorder(BevelBorder.RAISED);

	JPanel jn = new JPanel();
        jn.setLayout(new GridLayout(3,1,0,0));

	JPanel jn0 = new JPanel();
        jn0.setLayout(new FlowLayout());
	jn.add(jn0);

        jbSave = new JButton("Save");
        jbSave.addActionListener(this);
        jn0.add(jbSave);

	jtFile = new JTextField(10);
	jtFile.setEditable(true);
	jn0.add(jtFile);

	jlUnit = new JLabel("___");
	jn0.add(jlUnit);

	jcValue = new JComboBox(getNames());
	Vlist = 8;    // Stress
	jcValue.setSelectedIndex(Vlist);
        jcValue.addActionListener(this);
        jn0.add(jcValue);

	if (DEBUG) {
            jbDebug = new JButton("D");
            jbDebug.addActionListener(this);
            jn0.add(jbDebug);
        }
	//--

	JPanel jn1 = new JPanel();
        jn1.setLayout(new FlowLayout());
	jn.add(jn1);

        jbSet = new JButton("Set");
        jbSet.addActionListener(this);
        jn1.add(jbSet);

	jn1.add(new JLabel("Gauge:"));

	jtBant = new JTextField(4);
	jtBant.setEditable(true);
	jn1.add(jtBant);

	jn1.add(new JLabel("Length:"));

	jtLeng = new JTextField(7);
	jtLeng.setEditable(true);
	jn1.add(jtLeng);

	jn1.add(new JLabel("Copper:"));

	jtCopp = new JTextField(4);
	jtCopp.setEditable(true);
	jn1.add(jtCopp);
	//--

	JPanel jn2 = new JPanel();
        jn2.setLayout(new FlowLayout());
	jn.add(jn2);

	jtData = new JTextField(42);
	jtData.setEditable(false);
	jn2.add(jtData);

	add(jn, "North");
        jn.setBorder(bo);
	//-----

	JPanel jw = new JPanel();
        jw.setLayout(new GridLayout(5,1,0,0));

	jbGrap = new JButton("Graph");
        jbGrap.addActionListener(this);
        jw.add(jbGrap);

	jbWire = new JButton("Wire");
        jbWire.addActionListener(this);
        jw.add(jbWire);

	jbCent = new JButton("Cent");
        jbCent.addActionListener(this);
        jw.add(jbCent);

        jbPm = new JButton("< >");
        jbPm.addActionListener(this);
        jw.add(jbPm);

        jbMm = new JButton("> <");
        jbMm.addActionListener(this);
        jw.add(jbMm);

        add(jw, "West");
        jw.setBorder(bo);
	//-----

	JPanel je = new JPanel();
        je.setLayout(new GridLayout(5,1,0,0));

	jbSof = new JButton(Offon[ON]);
        jbSof.addActionListener(this);
	jbSof.setBorder(new TitledBorder("wire"));
        je.add(jbSof);

	jbOnoff = new JButton(Offon[ON]);
        jbOnoff.addActionListener(this);
	jbOnoff.setBorder(new TitledBorder("data"));
        je.add(jbOnoff);

        jbType = new JButton("Type");
        jbType.addActionListener(this);
        je.add(jbType);

	jspWire = new JSpinner();
	jspWire.addChangeListener(this);
	jspWire.setBorder(new TitledBorder("middle"));
	je.add(jspWire);

	jspBass = new JSpinner();
	jspBass.addChangeListener(this);
	jspBass.setBorder(new TitledBorder("bass"));
	je.add(jspBass);

        add(je, "East");
        je.setBorder(bo);
	//-----

	JPanel js = new JPanel();
        js.setLayout(new GridLayout(2,1,0,0));

	JPanel js0 = new JPanel();
        js0.setLayout(new GridLayout(1,1,0,0));
	js.add(js0);

	JPanel js1 = new JPanel();
        js1.setLayout(new GridLayout(1,4,0,0));
	js.add(js1);

	jsbWid = new JScrollBar(JScrollBar.HORIZONTAL,0, KB, 0, KB);
	jsbWid.setUnitIncrement(1);
	jsbWid.setBlockIncrement(Oct);
        jsbWid.addAdjustmentListener(this);
        js0.add(jsbWid);

        jbItype = new JButton(" Curve ");
        jbItype.addActionListener(this);
        js1.add(jbItype);

	jbTune = new JButton("Tune");
        jbTune.addActionListener(this);
        js1.add(jbTune);

	jlBeat = new JLabel("___", JLabel.RIGHT);
	js1.add(jlBeat);

	jslBeat = new JSlider(0, 30, 10);
        jslBeat.setPaintTicks(true);
        jslBeat.setPaintLabels(false);
        jslBeat.setMajorTickSpacing(10);
        jslBeat.setMinorTickSpacing(5);
        jslBeat.addChangeListener(this);
        js1.add(jslBeat);

        add(js, "South");
        js.setBorder(bo);
	//-----

	JScrollPane jsp = new JScrollPane(PG);
	add(jsp, "Center");
        PG.addMouseListener(this);
        PG.addMouseMotionListener(this);
    }

    public void setenabled() {
	boolean sv = (!STRINGsw || VALUEsw);
	boolean wi = (DISP == WIRE);
	jspBass.setEnabled(wi && sv);
	jspWire.setEnabled(wi && sv);
	jbType.setEnabled(wi);
	jbOnoff.setEnabled(wi);
	jbSof.setEnabled(wi);
	jcValue.setEnabled(wi && !VALUEsw);

	boolean gp = (DISP == GRAP);
	boolean cn = (DISP == CENT);
	jbPm.setEnabled(gp || cn);
	jbMm.setEnabled(gp || cn);
    }
    public void init_minmax() {
	MX = new minMax[9];

	MX[0] = new minMax(0, 300.0, "[Kgf]", "(GTension)");
	MX[1] = new minMax(0, 30.0, "[#]", "Gauge");
	MX[2] = new minMax(0, 4.0, "[mm]", "Copper.Dia");
	MX[3] = new minMax(0, 10.0, "[mm]", "All.Dia");
	MX[4] = new minMax(0, 20.0, "[g/cm3]", "Density");
	MX[5] = new minMax(0, 3000.0, "[N]", "Tension");
	MX[6] = new minMax(0, 100.0, "[%]", "Load");
	MX[7] = new minMax(0, 20.0, "[mm]", "Elongation");
	MX[8] = new minMax(0, 300.0, "[kgf/mm2]", "Stress");
    }
    public String[] getNames() {
	String[] str = new String[MX.length];

	for (int i = 0; i < MX.length; i++)
	    str[i] = MX[i].getName();

	return str;
    }
    //
    public class minMax {
	private double min;
        private double max;
        private String unit;
        private String name;

        public minMax() {
            setMinMax(0.0, 0.0, "", "");
        }
        public minMax(double m, double x, String u, String n) {
            setMinMax(m, x, u, n);
        }
        public void setMinMax(double m, double x, String u, String n) {
            min = m;
            max = x;
            unit = u;
            name = n;
        }
        public double getMin() {
            return min;
        }
        public double getMax() {
            return max;
        }
        public String getUnit() {
            return unit;
        }
        public String getName() {
            return name;
        }
    }
    //
    public void actionPerformed(ActionEvent e){
	if (e.getSource() == jbPm) // <>
	    set_wide(true);
	else if (e.getSource() == jbMm) // ><
	    set_wide(false);

	else if (e.getSource() == jbGrap)
	    setGrap();
	else if (e.getSource() == jbWire)
	    setWire();
	else if (e.getSource() == jbCent)
	    setCent();
	else if (e.getSource() == jbTune)
	    setTune();

	else if (e.getSource() == jbType)
	    set_type();

	else if (e.getSource() == jbSof)
	    set_string_onoff();
	else if (e.getSource() == jbOnoff)
	    set_value_onoff();

	else if (e.getSource() == jbSet)
	    set_value();

	else if (e.getSource() == jcValue) // wire_data
	    select_value();
	else if (e.getSource() == jbSave)
	    save_data();

	else if (e.getSource() == jbDebug) // for DEBUG
	    write_debug();

	else if (e.getSource() == jbItype) // curve|straight
	    set_itype();

	set_fname();
	setenabled();
	PG.repaint();
    }
    public void mousePressed(MouseEvent m) {
        MOUSWsw = !MOUSWsw;
	PG.repaint();
    }
    public void mouseMoved(MouseEvent m) {
        if (MOUSWsw) {
            PG.set_mx(m.getX());
            PG.repaint();
        }
    }
    public void mouseReleased(MouseEvent m) {}
    public void mouseDragged(MouseEvent m) {}
    public void mouseEntered(MouseEvent m) {}
    public void mouseExited(MouseEvent m) {}
    public void mouseClicked(MouseEvent m) {}
    public void adjustmentValueChanged(AdjustmentEvent e) {
	if (e.getSource() == jsbWid) { // >|<
	    switch (DISP) {
	    case GRAP:
	    case CENT:
		STT = jsbWid.getValue(); // 岼⤵
		break;
	    case WIRE:
		WIDE = jsbWid.getValue(); // 
		break;
	    default:
	    }
	}

	PG.repaint();
    }
    public void stateChanged(ChangeEvent e) {
	int dx = WSTT/100;
	if (e.getSource() == jspBass) // ObjectValueͥintѴ
	    XBS = SBS+dx*object2int(jspBass.getValue());

	if (e.getSource() == jspWire)
	    XMD = SMD+dx*object2int(jspWire.getValue());

	if (e.getSource() == jslBeat) {
	    if (!CURVEsw)
		set_curve(); // curve
	}

	PG.repaint();
    }
    public void set_itype() {
        CURVEsw = !CURVEsw;
        jbItype.setText(CURVEsw ? "Straight" : " Curve ");

	SC.setIsw(CURVEsw);
	CALCsw = false;
    }
    public void save_data() {
	PrintWriter pw = null;
        String fn = jtFile.getText().trim();
	fn = fn.equals("") ? get_fname() : fn;

	String data = get_save_data();
	int[] head = make_header();
	boolean mwd = (DISP == WIRE && VALUEsw);

        try {
	    pw = new PrintWriter(new FileWriter(fn));

	    if (mwd) { // .mwd
		String val = head[0]+","+head[1]+","+head[2];
		pw.println(val);
	    } else {
		pw.println("# Design data");
		pw.println("# "+data);
		pw.println(KB+" 2");
	    }

	    for (int i = 0; i < KB; i++) {
		if (mwd){ // .mwd
		    if (i >= head[0] && i < BHW) // 2ܸ
			writeGenData(pw, i);
		    writeGenData(pw, i);
		} else
		    pw.println(get_save_value(i));
	    }

	    pw.close();
	    Error = "";
	} catch(IOException ex) {
            Error = "File write error.";
        }
    }
    public void writeGenData(PrintWriter pw, int nw) {
	pw.print(dform(TK.getLength(nw))+",");
	pw.print(TK.getGauge(nw)+",");
	pw.print((float)TK.getCopper(nw));
	/* for եƥԥ
	if (G[nw].isCsw()) {
	    pw.print(","+G[nw].getDensity());
	    pw.print(","+G[nw].getWpitch());
	}
	*/ 
	pw.println();
    }
    public int[] make_header() {
	int sig = 10;              // 2ܸ sig~bhw
	int[] data = new int[3];

	data[0] = sig;             // monoEnd
	data[1] = (BHW-sig)*2+sig; // woundEnd
	data[2] = BHW+1;           // plainKey

	return data;
    }
    public String get_save_data() {
	String n = VALUEsw ? "mwd" : get_save_list()+get_save_unit();
	switch (DISP) {
	case GRAP:
	    return "Inharmonicity[cent]";
	case WIRE:
	    return n;
	case CENT:
	    return "ppd";
	default:
	    return "";
	}
    }
    public String get_save_list() {
	return MX[Vlist].getName();
    }
    public String get_save_unit() {
	return MX[Vlist].getUnit();
    }
    public String get_save_value(int key) {
	switch (DISP) {
	case GRAP:
	    return dform(getInha(key));
	case WIRE:
	    return dform(get_data(Vlist, key));
	case CENT:
	    //return dform(get_icent(key, 1));
	    return dform(get_cent(key));
	default:
	    return "";
	}
    }
    public void set_string_onoff() {
	STRINGsw = !STRINGsw;
	jbSof.setText(STRINGsw ? Offon[OFF] : Offon[ON]);
    }
    public void set_value_onoff() {
	VALUEsw = !VALUEsw;
	jbOnoff.setText(VALUEsw ? Offon[OFF] : Offon[ON]);
    }
    public String get_fname() {
	String f = VALUEsw ? "design.mwd" : "design.dat";
	switch (DISP) {
	case GRAP:
	    return "design.icd";
	case WIRE:
	    return f;
	case CENT:
	    return "design.ppd";
	default:
	    return "?";
	}
     }
    public void set_fname() {
	jtFile.setText(Error.equals("") ? get_fname() : Error);
	String u = VALUEsw ? "(Wire)" : get_save_unit();

	switch (DISP) {
	case GRAP:
	case CENT:
	    jlUnit.setText("[cent]");
	    break;
	case WIRE:
	    jlUnit.setText(u);
	    break;
	default:
	    jlUnit.setText("");
	}
    }
    public double get_beat_value() { // tuning_beats
	return jslBeat.getValue()/10.0;
    }
    public void select_value() { // wire_data
	Vlist = jcValue.getSelectedIndex();
    }
    public void set_value() {
	int key = PG.get_keyn();
	key_input(key);
	CALCsw = !(isBasekey(key) && !CALCsw);
        /* ϥ0,bend,a49,c88Ϸ׻ԥ */
    }
    public void set_type() {
	UPGPsw = !UPGPsw;
    }
    public void setGrap() {
	DISP = GRAP;
	set_scrollbar(true);
    }
    public void setWire() {
	DISP = WIRE;
	set_scrollbar(false);
    }
    public void setCent() {
	DISP = CENT;
	set_scrollbar(true);
    }
    public void setTune() {
	TUNEsw = !TUNEsw;
	jbTune.setText(TUNEsw ? "Clear" : "Tune");

	if (TUNEsw)
	    start_run();
	else
	    reset_cent();
    }
    public void reset_cent() {
	for (int i = 0; i < KB; i++)
	    set_cent(i, 0);
    }
    public void set_curve() {
        double[] tmp = TF2.getCurve(get_beat_value());

        for (int i = 0; i < KB; i++)
            set_cent(i, tmp[i]);
    }
    public void set_scrollbar(boolean sw) {
	if (sw) {
	    jsbWid.setValues(STT, END, 0, KB);
	    jsbWid.setUnitIncrement(1);
	    jsbWid.setBlockIncrement(Oct);
	} else {
	    jsbWid.setValues(WIDE, 100, WSTT/2, WSTT*5);
	    jsbWid.setUnitIncrement(WSTT/4);
	    jsbWid.setBlockIncrement(WSTT/2);
	}
    }
    public void set_wide(boolean sw) {
	switch (DISP) {
	case GRAP:
	    END = sw ? END*2 : END/2;
	    if (END > KB)
		END = KB;
	    else if (END < Ocu)
		END = Ocu;
	    if (END+STT > KB)
		STT = KB-END;
	    jsbWid.setValues(STT, END, 0, KB);
	    break;
	case WIRE:
	    break;
	case CENT:
	    PG.pm_level(sw);
	    break;
	default:
	}
    }
    public void key_input(int key) {
	String lng = jtLeng.getText();
        String gau = jtBant.getText();
        String cop = jtCopp.getText();
	Error = "";

	try {  // length
	    double lnum = Double.parseDouble(lng);
            if (lnum < 0)
                jtLeng.setText("?");
            else {
		if (isBasekey(key))
		    TG[key].setLength(lnum);
                TK.setLength(key, lnum);
                CALCsw = false;
            }
        }
	catch(NumberFormatException e) {
            Error = "Length ?";
        }

	try {  // gauge
	    double gaug = Double.parseDouble(gau);
	    if (gaug < TK.getBanteMin() ||
		gaug > TK.getBanteMax())  // ɲgeter
		jtBant.setText("over");
            else {
		if (isBasekey(key))
		    TG[key].setGauge(gaug);
                TK.setGauge(key, gaug);
                CALCsw = false;
            }
        }
        catch(NumberFormatException e) {
            Error = "Gauge ?";
        }

        try {  // copper
	    double copp = Double.parseDouble(cop);
	    copp = (copp <= 0) ? 0 : SC.get_Copper(copp);
	    if (isBasekey(key))
		TG[key].setCopper(copp);
	    TK.setCopper(key, copp);
            CALCsw = false;
        }
        catch(NumberFormatException e) {
            Error = "Copper ?";
        }
    }

    public void init_set() {
        STT = 0;        // log̥λϥޥΰ
	END = KB;       // " 
	WIDE = WSTT;
	XBS = SBS;
	XMD = SMD;
	Error = "";
	set_fname();
    }
    public void init_wire() {
	TK = new TuningKeyboard(KB); //
	TG = new Wire[KB];
	for (int i = 0; i < KB; i++)
	    TG[i] = new Wire(i);

	TG[BSTT].setWire(1020, 24.5, 2.6);
	TG[BEND].setWire(1090, 17.0, 0.2);
	TG[A49].setWire(410, 15.5, 0);
	TG[C88].setWire(52, 12, 0);

	SC = new ScaleCalculation(BSTT, BEND, A49, C88, TK, TG, TF2, CURVEsw);
    }
    public void set_wire() {
	SC.calcWire();
	CALCsw = true;
    }
    public double calc_gtension() {
	double ten = 0, all = 0;
	int chord = 3;
	int sig = 10;

	for (int i = 0; i < KB; i++) {
	    ten = TK.getGTension(i);
	    if (i >= sig && i < BHW) // 2ܸ
		all += ten;
	    all += TK.isCopper(i) ? ten : ten*chord;
	}

	return all;
    }
    public double get_data(int v, int k) {
	switch(v) {
	case 0:
	    return TK.getGTension(k);
	case 1: 
	    return TK.getGauge(k);
	case 2: 
	    return TK.getCopper(k);
	case 3:
	    return TK.getAlld(k);
	case 4:
	    return TK.getDensity(k);
	case 5: 
	    return TK.getTension(k);
	case 6:
	    return TK.getLoad(k);
	case 7:
	    return TK.getElongation(k);
	case 8:
	    return TK.getStress(k);
	default:
	    return 1;
	}
    }
    public boolean isBasekey(int key) {
	return (key == BSTT || key == BEND ||
		key == A49 || key == C88);
    }
    public String dform(double n) {
        return df1.format((float)n);
    }
    public String dform2(double n) {
        return df2.format((float)n);
    }
    public double log_base(double value) {
        return Math.log(value)/MLOG10;
    }
    public int object2int(Object val) {
	return ((Integer)val).intValue();
    }
    public void write_debug() { //for DEBUG
        for (int i = 0; i < KB; i++)
	    System.out.println(i+":"+getInha(i)+", "+get_cent(i));
    }
    //-----

    public void start_run() {
	if (CURVEsw) { // straight
	    TK.startRun(get_beat_value());

	    thr = new Thread(this);
	    thr.start();
	} else { // curve
	    set_curve();
	    PG.repaint();
	}
    }
    public void run() {
	jbTune.setEnabled(false);

        Error = "";
        int stime = 100;
        try {
            while (TK.isRun()) {
		thr.sleep(stime);
		PG.repaint();
            }
        } catch (InterruptedException e) {
            Error = "Run error."+e;
        }
        finally {
	    jbTune.setEnabled(true);
        }

        thr = null;
    }

    public void add_cent(int kn, double ce) {
	TK.addCent(kn, ce);
    }
    public void set_cent(int kn, double ce) {
	TK.setCent(kn, ce);
    }
    public double get_cent(int kn) {
        return TK.getCent(kn);
    }
    public double get_icent(int kn, int pr) {
	return TK.getIcent(kn, pr);
    }
    public double get_freq(int kn) {
	return TK.getFreq(kn);
    }
    public double get_ifreq(int kn, int pr) {
	return TK.getIfreq(kn, pr);
    }
    public double getInha(int kn, int pr) {
        return TK.getInha(kn, pr);
    }
    public double getInha(int kn) {
	return TK.getInha(kn);
    }
    //--------

    public class PaintGraph extends JPanel {
	int W, H, H2;
	int CX = 50;       // centΨ
	int Fns = 14;      // font
        int DOT = 4;
        int DSF = DOT/2;
	int NK = A49;
	double DY;         // ⤵Ψ
	double DK;         // wide̤ǤΦ
	double DX;         // KBƤǤΦ
	double DC;         // centͤΨ
	double DW;         // ˡإѹ
	double BHL = 14.5; // ϥޡ bass
	double MHL = 13.5; // " mid
	double HHL = 13;   // " hi
	double SPH = 32;   // Ŵʬ hi
	double SPL = 46;   // " low

        Font FNT = new Font("serif", Font.PLAIN, Fns);
	FontMetrics fm = getFontMetrics(FNT);

	final static double CBY = 1000; // [mm]ȹ碌٤centΨ
	final String[] Keynm = 
        {"A","B","H","C","Cis","D","Es","E","F","Fis","G","Gis"};
	final Color BG = new Color(00,180,255); // stress
	final Color RB = new Color(120,0,180);  // freq.

        public PaintGraph() {
            setBackground(Color.black);
        }
        public void paintComponent(Graphics g) {
	    Dimension d = getSize();
	    W = d.width;
	    H = d.height;
	    H2 = H/2;

	    DC = H2/(double)CX;
	    set_dwide();
	    set_dot();
	    super.paintComponent(g);

	    if (!CALCsw)
		set_wire();

	    switch (DISP) {
	    case GRAP:
		disp_wire(g);
		break;
	    case WIRE:
		if (!STRINGsw || VALUEsw)
		    disp_strings(g);
		if (!VALUEsw) {
		    disp_data(g, 0);
		    if (Vlist > 0)
			disp_data(g, Vlist);
		}
		break;
	    case CENT:
		disp_cent(g);
		break;
	    default:
            }

	    write_value();
	    disp_beat();
	}
	public void set_dwide() {
	    DK = (W-1)/(double)END;
	    DX = (W-1)/(double)KB;
	    DW = (W-1)/(double)WIDE;
	}
        public void pm_level(boolean sw) {
            CX = sw ? CX*2 : CX/2;
            if (CX > 100)
                CX = 5;
	    if (CX > 13 && CX < 25)
		CX = 25;
	    if (CX < 13 && CX > 5)
		CX = 10;
	    if (CX < 5)
		CX = 100;
	}
	public void set_dot() {
	    int min = 1, max = 8;

	    DOT = (int)DK;
	    if (DOT > max)
		DOT = max;
	    if (DOT < min)
		DOT = min;

	    DSF = DOT/2;
	}
        public void set_mx(int x) {
	    switch (DISP) {
	    case GRAP:
	    case CENT:
		NK = (int)(x/DK)+STT;
		break;
	    case WIRE:
		NK = (int)(x/DX);
		break;
	    default:
            }

            if (NK < 0)
                NK = 0;
            if (NK >= KB)
                NK = KB-1;
        }
	public int get_keyn() {
	    return NK;
	}

	public void write_value() {
	    jtLeng.setText(dform2(TK.getLength(NK)));
	    jtBant.setText(dform2(TK.getGauge(NK)));
	    jtCopp.setText(dform2(TK.getCopper(NK)));

	    StringBuffer sb = new StringBuffer(64);
	    sb.append((NK+1)+"(");
	    sb.append(Keynm[NK%Oct]+")");
	    switch (DISP) {
	    case GRAP:
		sb.append(dform2(TK.getFreq(NK))+"[Hz] (1/");
		sb.append(dform2(SC.getSPoint(NK))+"=");
		sb.append(dform2(TK.getLength(NK)/SC.getSPoint(NK))+"[mm])");
		sb.append(dform(getInha(NK))+"[cent](");
		sb.append(dform2(getInha(NK)*1.18)+"max)"); // +18%
		double tmp = TG[NK].getInhar();
		if (tmp > 0)
		    sb.append(" temp:"+dform(tmp)); // for Debug?	
		break;
	    case WIRE:
		sb.append(dform2(TK.getFreq(NK))+"[Hz] ");
		if (VALUEsw)
		    sb.append(TK.toString(NK));
		else {
		    sb.append(dform2(TK.getGTension(NK))+"[Kgf] ");
		    if (Vlist > 0)
			sb.append(dform2(get_data(Vlist, NK))+
				  MX[Vlist].getUnit());
		}
		break;
	    case CENT:
		sb.append(dform2(get_ifreq(NK,1))+"[Hz] ");
		//sb.append(dform2(get_icent(NK,1))+"[cent]");
		sb.append(dform2(get_cent(NK))+"[cent]");
		break;
	    default:
            }

	    jtData.setText(sb.toString());
	}

        public void disp_cmemori(Graphics g) {
            g.setColor(Color.gray);   // center line
            g.drawLine(0, H2, W-1, H2);

            int dm = CX/5, mp, mm, pl, ml;
	    String ps, ms;
	    double dd;
            for (int i = 1; i <= 5; i++) { // cent memori
		dd = dm*DC*i;
                mp = (int)(H2-dd);
                mm = (int)(H2+dd);
		g.setColor(Color.blue);
                g.drawLine(0, mp, W-1, mp);
                g.drawLine(0, mm, W-1, mm);

		ps = ""+dm*i;
		pl = fm.stringWidth(ps);
		ms = "-"+dm*i;
		ml = fm.stringWidth(ms);
		g.setColor(Color.orange);
		g.drawString(ps, W-pl, mp+Fns-3);
		g.drawString(ms, W-ml, mm);
            }
        }
	public void disp_beat() {
	    jlBeat.setText("Beat:"+dform(get_beat_value()));
	}
        public void disp_cent(Graphics g) {
	    disp_keys(g);
	    disp_cmemori(g);

	    int x, y, ii;
	    for (int i = 0; i < END; i++) {
		ii = i+STT;
		x = (int)(DK*i);
                //y = (int)(H2-get_icent(ii, 1)*DC-DSF);
		y = (int)(H2-get_cent(ii)*DC-DSF);

		set_color(g, ii);
		g.fillOval(x, y, DOT, DOT);
	    }
        }

	public void disp_strings(Graphics g) {
	    double botom, bass = get_bass_length()*DW;
	    double ys = bass/8+30*DW;
	    double xs = XMD*DW, xb = XBS*DW;
	    double ml = (get_spx(BHW+MHW-1)-xs)/(double)(MHW-1);
	    double bl = 20*DW;
	    double x, dl;

	    ys = UPGPsw ? H-ys : ys;
	    botom = UPGPsw ? H-bass : bass;

	    for (int i = C88; i >= 0; i--) {
		x = get_spx(i);
		if (i < BHW)
		    dl = xb+i*bl;
		else if (i < BHW+MHW)
		    dl = xs+(i-BHW)*ml;
		else
		    dl = x;

		set_color(g, i);
		disp_ps_line(g, x, ys, dl, botom, i, DW);
	    }

	    g.setColor(Color.yellow); // s.point
	    g.drawLine(1, (int)ys, W-1, (int)ys);
	}
	public void set_color(Graphics g, int k) {
	    g.setColor((k == NK) ? get_cursol_color(k) : get_wire_color(k));
	}
	public Color get_wire_color(int k) {
	    return TK.isCopper(k) ? Color.pink : Color.white;
	}
	public Color get_cursol_color(int k) {
	    return isBasekey(k) ? Color.magenta : Color.green;
	}
 	public int get_bass_length() {
	    int max = 0;
	    double l;
	    for (int i = 0; i < BHW; i++) {
		l = TK.getLength(i);
		if (max < l)
		    max = (int)l;
	    }
	    return max+100;
	}
	public double get_spx(int n) {
	    double sft = 100;

	    if (n < BHW)
		return DW*(BHL*n+sft);
	    else if (n < BHW+MHW)
		return DW*(BHL*BHW+SPL+MHL*(n-BHW)+sft);
	    else
		return DW*(BHL*BHW+SPL+MHL*MHW+SPH+HHL*(n-BHW-MHW)+sft);
	}
	public void disp_ps_line(Graphics g, double ax, double ay, 
				 double bx, double by, int i, double dh) {
	    Point xy = fnp(ax, ay, bx, by, 
			   dh*TK.getLength(i)*(1-1.0/SC.getSPoint(i)));
	    g.drawLine((int)ax, (int)ay, xy.x, xy.y);

	    Point yx = fnr(ax, ay, bx, by, 
			   dh*TK.getLength(i)/SC.getSPoint(i));
	    g.drawLine((int)ax, (int)ay, yx.x, yx.y);
	}
	public Point fnp(double ax, double ay, double bx, double by, double l) {
	    double xsa = bx-ax;
	    double ysa = by-ay;
	    double xy2 = Math.sqrt(Math.pow(xsa,2)+Math.pow(ysa,2));
	    Point p = new Point((int)(ax+xsa*l/xy2), (int)(ay+ysa*l/xy2));
	    return p;
	}
	public Point fnr(double ax, double ay, double bx, double by, double l) {
	    double xsa = ax-bx;
	    double ysa = ay-by;
	    double xy2 = Math.sqrt(Math.pow(xsa,2)+Math.pow(ysa,2));
	    Point r = new Point((int)(ax+xsa*l/xy2), (int)(ay+ysa*l/xy2));
	    return r;
	}

	public void disp_wire(Graphics g) {
	    disp_keys(g);
	    disp_logscale(g); // set DY
	    disp_logunit(g);
	    disp_grade(g);

	    int x, f, c, s, h, m, ch, tp, ii;
	    double x0, tmp;
	    for (int i = 0; i < END; i++) {
		x0 = DK*i;
		x = (int)x0;
		ii = i+STT;

		f = log_y(TK.getFreq(ii));
		g.setColor(RB);
		g.fillOval(x, f, DOT, DOT); // ȿ

		c = log_y(TK.getLength(ii));
		g.setColor(get_wire_color(ii));
		g.fillOval(x, c, DOT, DOT); // Ĺ

		s = log_y(TK.getLength(ii)/SC.getSPoint(ii));
		g.setColor(Color.yellow);
		g.fillOval(x, s, DOT, DOT); // Ǹ

		h = log_y(CBY*SC.getIbase(ii));
		g.setColor(Color.lightGray);
		g.fillOval(x, h, DOT, DOT); // ।ϡ˥ƥ

		m = log_y(CBY*SC.getIbase(ii)*1.18);
		g.setColor(Color.gray);
		g.fillOval(x, m, DOT, DOT); // 祤ϡ˥ƥ

		ch = log_y(CBY*getInha(ii));
		g.setColor(Color.cyan);
		g.fillOval(x, ch, DOT, DOT); // ϡ˥ƥ

		tmp = TG[ii].getInhar();
		if (tmp > 0) {
		    tp = log_y(CBY*tmp);
		    g.setColor(Color.red);
		    g.fillOval(x, tp, DOT, DOT); // tempG
		}

		if (ii == NK) {  // 
		    int cx = (int)(x0+DSF);
		    g.setColor(get_graph_cursol(ii));
	 	    g.drawLine(cx, 0, cx, H-1);
		}
	    }
	}
	public int log_y(double val) {
	    return (int)(H-log_base(val)*DY-DSF);
	}
	public Color get_graph_cursol(int k) {
	    return isBasekey(k) ? Color.magenta : Color.white;
	}

	public void disp_logscale(Graphics g) {
	    int miny = 1, maxy = 100000, y;
	    DY = H/log_base(maxy/miny);
	    int cl;
	    String cs;

	    for (int i = miny; i <= maxy; i *= 10) {
		y = (int)(H-log_base(i)*DY);

		g.setColor(Color.blue);
		g.drawLine(0, y, W-1, y);

		g.setColor(Color.orange);  // left sclae
		g.drawString(""+i, 1, y);

		cs = dform(i/CBY);
		cl = fm.stringWidth(cs);
		g.setColor(Color.green);   // right scale
		g.drawString(cs, W-cl, y);
	    }
	}
	public void disp_keys(Graphics g) {
	    int x, ii;
	    for (int i = 0; i < END; i++) {
                x = (int)(DK*i+DSF);
		ii = i+STT;

		if (ii%Oct == 0) {
		    g.setColor(Color.blue);
		    g.drawLine(x, 0, x, H-1);

		    g.setColor(Color.red);
		    g.drawString((ii+1)+"A", x, H-Fns);
		}
	    }
	}
	public void disp_grade(Graphics g) {
	    String grd;
	    int ln;

	    if (CURVEsw) {
		LeastSquares ls = SC.getLSM(TK.getG()); // 49-88
		grd = "grade:"+dform(ls.getLSMB());
		ln = W-fm.stringWidth(grd);
	    
	    } else {
		grd = TF2.toString();
		ln = W-fm.stringWidth(grd);
	    }

	    g.setColor(Color.green);
	    g.drawString(grd, ln/2, Fns*2);
	}
	public void disp_logunit(Graphics g) {
	    int y0 = Fns, x0 = 1;

	    String hs = "[Hz]";
	    g.setColor(RB);
	    g.drawString(hs, x0, y0);

	    String sp = "[mm]";
	    x0 += fm.stringWidth(hs);
	    g.setColor(Color.yellow);
	    g.drawString(sp, x0, y0);

	    String s1 = "[m";
	    x0 += fm.stringWidth(sp);
	    g.setColor(Color.pink);
	    g.drawString(s1, x0, y0);

	    String s2 = "m]";
	    x0 += fm.stringWidth(s1);
	    g.setColor(Color.white);
	    g.drawString(s2, x0, y0);

	    String cs = "[cent]";
	    int cl = fm.stringWidth(cs);
	    g.setColor(Color.cyan);
	    g.drawString(cs, W-cl, y0);

	    g.setColor(Color.lightGray);
	    g.drawString(cs, W-cl*2, y0);
	}
	public void disp_data(Graphics g, int val) {
	    int max = (int)MX[val].getMax();
	    double dh = (H-1)/(double)max;
	    int x1, y1;
	    int dot = (int)DX;
	    int sft = (int)(DX/2);

	    for (int i = 0; i < KB; i++) {
		y1 = (int)(H-get_data(val, i)*dh);
		x1 = (int)(DX*i+sft);

		g.setColor((i == NK) ?
			   get_cursol_color(i) : get_memori_color(val));
		g.fillOval(x1-sft, y1-sft, dot, dot);
	    }

	    disp_memori(g, dh, max, val);
	    disp_ton(g);
	}
	public void disp_ton(Graphics g) {
	    String st = dform2(calc_gtension()/1000)+"[ton]";
	    int ln = fm.stringWidth(st);

	    g.setColor(Color.red);
	    g.drawString(st, W-(ln+Fns), Fns);
	}
	public void disp_memori(Graphics g, double dw, int max, int val) {
	    int y, l = 5;
	    int stp = get_steps(max);

	    g.setColor(get_memori_color(val));
	    for (int i = 0; i < max; i += stp) {
		y = (int)(H-i*dw-1);
		l = get_l(i, stp);

		if (val > 0)
		    g.drawLine(W-1, y, W-l, y);
		else
		    g.drawLine(1, y, l, y);
	    }
	}
	public int get_steps(int max) {
	    int stp = 1;

	    if (max > 1000)
		stp = 100;
	    else if (max >= 100)
		stp = 10;

	    return stp;
	}
	public int get_l(int i, int stp) {
	    int l = 5;

	    if (i%(stp*10) == 0)
		l = 15;
	    else if (i%(stp*5) == 0)
		l = 10;
	    else
		l = 7;

	    return l;
	}
	public Color get_memori_color(int val) {
	    return (val > 0) ? BG : Color.orange;
	}
    }
}
