/* $Id: TuningKeyboard.java,v 1.6 2013/12/04 23:53:10 mit Exp $ */
/* TuningKeyboard.java v0.2.2 */

public class TuningKeyboard implements Runnable
{
    private boolean Runsw;
    private int KB = 88;
    private int A37 = 37-1;
    private int A49 = 49-1;
    private int A61 = 61-1;
    private int Oct2;
    private double Beat = 1.0;
    private Wire[] G;
    private Thread thr;
    private Interval I = new Interval();

    final static int OCT = 12;
    final static int OCT2 = OCT*2;
    final static double E = 2.15*100000; // [N/mm^2]

    public TuningKeyboard() {}
    public TuningKeyboard(int kb) {
	KB = kb;
        G = new Wire[KB];

        for (int i = 0; i < KB; i++)
            G[i] = new Wire(i);

	Oct2 = I.getIinte(OCT2); // 27
    }
    //
    public void startRun(double be) {
        Beat = be;

        thr = new Thread(this);
        thr.start();

        Runsw = true;
    }
    public boolean isRun() {
        return Runsw;
    }
    public void stop() {
        if (thr != null) {
            thr = null;
            Runsw = false;
        }
    }
    public void run() {
        int stime = 10;              // ꡼׻

        try {
	    double sa = get_cent_val();
	    set_2octave(sa/OCT2, sa/2);
	    
	    for (int i = A61+1; i < KB; i++) {
		tuning(i, Oct2, true);
                thr.sleep(stime);
            }
            for (int i = A37-1; i >= 0; i--) {
		tuning(i, Oct2, false);
                thr.sleep(stime);
            }
	}
        catch (InterruptedException e) {
            System.out.println("Run error."+e);
        }

	Runsw = false;
    }
    public void set_2octave(double ds, double sf) {
	double a49d = getCent(A49);
	for (int i = 0; i <= OCT2; i++)
	    key_set(a49d, i+A37, ds*i-sf);
    }
    public double get_cent_val() {
	int k38 = 38-1;  // 2octave+1keycentФ
	int k62 = 62-1;
	Inter k2 = new Inter(k38, k62, Oct2);

	int k36 = 36-1;  // 2octave-1keycent
	int k60 = 60-1;
	Inter k1 = new Inter(k36, k60, Oct2);

 	double dc = 0.01;             // ѹ
	double up = 0, ds, sa, sb, d;
	do {
  	    ds = up/OCT2;
	    set_2octave(ds, up/2);            // Ф
	    tuneBeat(true, k2);               // +1keytune
	    sa = getCent(k62)-getCent(A61);   // +1key2octaveüκ

	    tuneBeat(false, k1);              // -1keytune
	    sb = getCent(A37)-getCent(k36);   // -1key2octaveüκ

	    d = (sa-ds > 0.6) ? dc*10 : dc;
	    up += d;
	} while (sa > ds && sb > ds);         // +1-1ɤ餫κ

	return up-d;                          // 1äset_2octaveԤ
    }
    public void tuning(int k, int in, boolean sw) {
        int nx = nextKey(k, (sw ? -I.getIvalue(in) : I.getIvalue(in)));
        if (nx < 0 || nx >= KB)
            return;

	Inter In = sw ? new Inter(nx, k, in) : new Inter(k, nx, in);
	tuneBeat(sw, In); // beat
    }
    public void key_set(double c49, int k, double dt) {
        setCent(k, c49);   // 濴ͥ˥å
        addCent(k, dt);    // θ糧
    }
    public int nextKey(int nowkey, int itr) {
        return nowkey+itr;
    }
    public double tuneBeat(boolean sw, Inter k) {
	double d, dc = 0.01;
	double mc = getBeats(k);
	double max = Beat;
	int wn;

        if (sw) {
            wn = k.getHkey();
            dc = dc;
        } else {
            wn = k.getLkey();
            dc = -dc;
        }

        double sa = 0.4;
	while (mc < max) {   // first
	    d = (max-mc > sa) ? dc*10 : dc;
	    addCent(wn, d);
	    mc = getBeats(k);
	}
	while (mc > max) {   // second
	    d = (max-mc > sa) ? dc*10 : dc;
	    addCent(wn, -d);
	    mc = getBeats(k);
	}

        return mc;
    }
    public double getBeats(Inter n) {
	return getIfreq(n.getHkey(), n.getLint())-
	    getIfreq(n.getLkey(), n.getHint());
    }
    //
    public void addCent(int kn, double ce) {
	G[kn].addCent(ce);
    }
    public void setCent(int kn, double ce) {
	G[kn].setCent(ce);
    }
    public double getCent(int kn) {
        return G[kn].getCent();
    }
    public double getIcent(int kn, int pr) {
	return getInha(kn, pr)+getCent(kn);
    }
    public double getFreq(int kn) {
	return G[kn].getFreq();
    }
    public double getIfreq(int kn, int pr) {
	return ctof(getFreq(kn)*pr, getInha(kn, pr));
    }
    public double getInha(int kn, int pr) {
        return G[kn].getInhar()*pr*pr;
    }
    public double getInha(int kn) {
	return G[kn].getInhar();
    }
    public double ctof(double f, double ce) {
	return f*Math.pow(2, ce/1200);
    }
    //
    public void setLength(int k, double le) {
        G[k].setLength(le);
    }
    public void setGauge(int k, double ga) {
        G[k].setGauge(ga);
    }
    public void setCopper(int k, double co) {
        G[k].setCopper(co);
    }
    public void setWire(int k, double le, double ga, double co) {
        G[k].setWire(le, ga, co);
    }
    public double getLength(int k) {
        return G[k].getLength();
    }
    public double getGauge(int k) {
        return G[k].getGauge();
    }
    public double getCopper(int k) {
        return G[k].getCopper();
    }
    public double getDensity(int k) {
	return G[k].getDensity();
    }
    public double getGTension(int k) {
        return G[k].getGTension();
    }
    public double getTension(int k) {
        return G[k].getTension();
    }
    public double getDia(int k) {
        return G[k].getDia();
    }
    public double getDiameter(int k) {
        return G[k].getDiameter();
    }
    public double getAlld(int k) {
        return G[k].getAlld();
    }
    public double getalld(int k) {
        return G[k].getalld();
    }
    public double getLoad(int k) {
        return G[k].getLoad();
    }
    public double getElongation(int k) {
        return G[k].getElongation(E);
    }
    public double getStress(int k) {
        return G[k].getStress();
    }
    public String toString(int k) {
        return G[k].toString();
    }
    public int getNum(int k) {
        return G[k].getKeynum();
    }
    public boolean isCopper(int k) {
        return G[k].isCopper();
    }
    public Wire[] getG() {
	return G;
    }
    public double getBanteMin() {
	return G[0].getBanteMin();
    }
    public double getBanteMax() {
	return G[0].getBanteMax();
    }
}