/* $Id: Wire.java,v 1.10 2013/12/04 23:52:00 mit Exp $ */
/* Wire.java v5.3 (for Design) */
public class Wire{
    private final int CENT = 1200;
    private final int METER = 1000;
    private final int CM = 10;
    private final double KM = 6.9906;        // Ƽȶκ̩ g/cm^3
    private final double WM = 9.0;           // ̩ g/cm^3
    private final double SM = 7.85;          // ̩ g/cm^3
    private final double BM = 8.97;          // ̩ g/cm^3
    private final double NG = 9.80665204821; // ϲ® kgw

    private static boolean csw = false;
    private static String sepa = ",";
    private static int monoEnd;  // OB
    private static int woundEnd; // DB
    private static int plainKey; // SB
    private static int nonKeys;  // FB
    private static int fortePsw; // CB
    private static int counter;
    private static int pitch = 440;

    private int keynum = 1;
    private double length;
    private double gauge;
    private double copper;
    private double density;
    private double wpitch;
    private double cent;
    private double freq;
    private double dia;
    private double ald; //
    private Bante b = new Bante();

    public static void setHeader(int[] data) {
	monoEnd = data[0];
	woundEnd = data[1];
	plainKey = data[2];
	if (data.length > 3) {
	    nonKeys = data[3];
	    fortePsw = data[4];
	}
	csw = (fortePsw > 0) ? true : false;
	pitch = (fortePsw > 0) ? 415 : 440;

	counter = 0;
    }

    public Wire() { }
    public Wire(double l, double g, double c) {
	setWire(l, g, c, 0, 0, 0);
    }
    public Wire(double l, double g, double c, double d) {
	setWire(l, g, c, d, 0, 0);
    }
    public Wire(double l, double g, double c, double d, double p, double ce) {
        setWire(l, g, c, d, p, ce);
    }
    public Wire(double[] w) {
	setWire(w[0], w[1] ,w[2], w[3], w[4], 0);
    }
    public Wire(Wire w) {
	setWire(w.getLength(), w.getGauge(), w.getCopper(),
		w.getDensity(), w.getWpitch(), w.getCent());
    }
    public Wire(int key) { // for Design 
	setWire(key, 0,0,0,0,0,0);
    }	
    public void setWire(int key, double l, double g, double c,
			double d, double p, double ce) { // for Design
	this.setGen(l, g, c, d, p, ce);

	keynum = key;
	resetWire();
	counter++;
    }
    public void setWire(double l, double g, double c,
			double d, double p, double ce) {
	this.setGen(l, g, c, d, p, ce);

	keynum = this.wtok(counter++);
	resetWire();
    }
    public void setGen(double l, double g, double c,
		       double d, double p, double ce) {
	length = l;
	gauge = g;
	copper = c;
	density = d;
	wpitch = p;
	cent = ce;
    }
    public void setWire(double l, double g, double c) { // for Design
	length = l;                               // ߷פǤѹ
	gauge = g;
	copper = c;

	resetWire();
    }
    public void resetWire() {
        freq = this.ctof();
        dia = this.getDiameter();
	ald = this.getAlld();
    }
    public void setLength(double l) {
	length = l;
	resetWire();
    }
    public void setGauge(double g) {
	gauge = g;
	resetWire();
    }
    public void setCopper(double c) {
	copper = c;
	resetWire();
    }
    public void setDensity(double d) {
	density = d;
    }
    public void setWpitch(double w) {
	wpitch = w;
    }
    public void setPitch(int a) {
	pitch = a;
    }
    public void setCent(double c) {
	cent = c;
	freq = this.ctof();
    }
    public void addCent(double c) {
	cent += c;
	freq = this.ctof();
    }
    public void setCsw(boolean sw) {
	csw = sw;
    }
    public boolean isCopper() {
	return (copper <= 0) ? false : true;
    }
    public boolean isCsw() {
	return csw;
    }

    public double[] getWire() {
	double[] w = {length, gauge, copper, density, wpitch, freq};

	return w;
    }
    public static int[] getHeader() {
	int[] h = {monoEnd, woundEnd, plainKey,	nonKeys, fortePsw};
    
	return h;
    }
    public double get_freq() { // 0~87 ?
	int key = keynum+1;
	return pitch*Math.pow(2, (key-49)/12.0);
    }
    public double ctof() {
	return this.get_freq()*Math.pow(2, cent/CENT);
    }
    public int wtok(int w) {
	return (w < woundEnd) ? 
	    ((w >= monoEnd) ? monoEnd+1+(w-monoEnd)/2 : w+1) :
	    w-woundEnd+plainKey;
    }

    public void setSeparater(String s) {
        sepa = s;
    }
    public String toString() {
        StringBuffer sb = new StringBuffer(48);
        sb.append((float)length).append(sepa);
        sb.append(gauge).append(sepa);
        sb.append((float)copper);
        if (density > 0) {
            sb.append(sepa).append(density);
            sb.append(sepa).append(wpitch);
        }
        sb.append("\n");

        return sb.toString();
    }
    public String getHeaderString() {
	StringBuffer sb = new StringBuffer(48);
        sb.append(monoEnd+","+woundEnd+","+plainKey);
	if (csw)
	    sb.append(","+nonKeys+","+fortePsw);
	sb.append("\n");

	return sb.toString();
    }
    public int getPitch() {
	return pitch;
    }
    public int getKeynum() {
	return keynum;
    }
    public int getCounter() {
	return counter;
    }
    public double getLength() {
	return length;
    }
    public double getGauge() {
	return gauge;
    }
    public double getCopper() {
	return copper;
    }
    public double getDensity() {
	return this.isCopper() ? getWDensity() : SM;
    }
    public double getWpitch() {
	return wpitch;
    }
    public double getCent() {
	return cent;
    }
    public double getFreq() {
	return freq;
    }
    public double getAlld() {
	return csw ? gauge : copper*0.94*2+dia;
    }
    public double getalld() { // for 100%
        return csw ? gauge : copper*2+dia;
    }
    public double getDia() {
	return dia;
    }
    public double getDiameter() {
	return csw ? gauge : b.getDiameter(gauge);
    }
    public double getWDensity() {
	if (csw) {
	    return (WM*Math.pow(gauge/CM,2)+
		    density*Math.pow(copper/CM,2)*
		    ((gauge+copper)/CM)*Math.PI*(CM/wpitch))/
		Math.pow(gauge/CM,2);
	} else {
	    double j = Math.pow(getAlld()/2,2);
	    double g = Math.pow(dia/2,2);
	    return (KM*(j-g)+SM*g)/j;
	}
    }
    public double getGTension() {
	return getTension()/NG;
    }
    public double getTension() {
	double di = csw ? dia : ald;
        double de = csw ? density : SM;
        de = this.isCopper() ? this.getDensity() : de;

	return Math.PI*de*METER*Math.pow(freq,2)*
	    Math.pow(length/METER,2)*Math.pow(di/METER,2);
    }
    public double getInhar() {
	if (this.isCopper()) {
	    double sm = csw ? density : SM;

	    return 3.3*Math.pow(CM,13)*Math.pow(dia/CM,2)/
		Math.pow(length/CM,4)/
		Math.pow(Math.pow(getTension()/Math.PI/(sm*METER),1.0/2)/
			 (length/METER)/(dia/METER),2)*(1+getAlld()/dia/CM);
	} else {
	    return 3.3*Math.pow(CM,13)*Math.pow(dia/CM,2)/
		Math.pow(length/CM,4)/Math.pow(freq,2);
	}
    }
    public double getLoad() {
        return getTension()/b.getFzn(gauge)*100.0;
    }
    public double getElongation(double e) {
        return getTension()*length/
            (Math.pow(dia,2)*Math.PI*e/4);
    }
    public double getStress() {
	return getGTension()/Math.pow(dia/20,2)*Math.PI/METER;
    }
    public double getBInhar(int p, double e) {
	return CENT*Math.log(1+p*p*getB(e))/(2*Math.log(2));
    }
    public double getB(double e) {
	return Math.pow(Math.PI,3)*e*Math.pow(dia,4)/
	    (64*getTension()*Math.pow(length,2)); // [N/mm^2]
    }
    public double getBanteMin() {
        return b.getMin();
    }
    public double getBanteMax() {
        return b.getMax();
    }
} 
